// get functions for major input elements
function getRaInputElement() { return document.getElementById('ra'); }
function getDecInputElement() { return document.getElementById('dec'); }
function getRadiusInputElement() { return document.getElementById('sr'); }
function getLookUpElement() { return document.getElementById('lookUp'); }
function getUrlInputElement() { return document.getElementById('url'); }

// get functions for major input values
function getRaInput() { return getRaInputElement().value; }
function getDecInput() { return getDecInputElement().value; }
function getRadiusInput() { return getRadiusInputElement().value; }
function getLookUpInput() { return getLookUpElement().value; }
function getUrlInput() { return getUrlInputElement().value; }

// set functions for major input values
function setRaInput(value) {
    getRaInputElement().value = value;
}
function setDecInput(value) {
    getDecInputElement().value = value;
}
function setRadiusInput(value) {
    getRadiusInputElement().value = value;
}
function setLookUpInput(value) {
    getLookUpElement().value = value;
}

function clear() {
    setRaInput('');
    setDecInput('');
    setRadiusInput('');
}

function setCoords(ra, dec, sr) {
    clear();
    setRaInput(ra);
    setDecInput(dec);
    setRadiusInput(sr);
}

function createExample(text, input1, input2, input3) {
    trExample = document.createElement('tr');
    aExample = document.createElement('a');
    tdExamplePrompt = document.createElement('td');
    tdExampleLink = document.createElement('td');
    
    tdExamplePrompt.innerHTML = text + '&nbsp;&nbsp;&nbsp;';
    aExample.innerHTML = 'RA = ' + input1 + ', Dec = ' + input2 + 
    ', Radius = ' + input3;
    aExample.href = 'javascript:setCoords("' + input1 + '", "' + input2 + 
    '", "' + input3 + '");';
    
    tdExampleLink.appendChild(aExample);
    trExample.appendChild(tdExamplePrompt);
    trExample.appendChild(tdExampleLink);
    return trExample;
}

function listExample(text, input1, input2, input3) {
    exampleElement = document.getElementById('example');
    exampleElement.appendChild(createExample(text, input1, input2, input3));
}

function loadExampleList() {
    exampleElementRoot = document.getElementById('example_list');
    exampleElement = document.createElement('table');
    exampleElement.id = 'example';
    exampleElementRoot.innerHTML = "";
    exampleElementRoot.appendChild(exampleElement);

    listExample('Crab Nebula', '05:34:31.97', '+22:00:52.1', '0.1');
    listExample('Cassiopeia A', '23:23:24', '+58:48.9', '0.1');
    listExample('Sagittarius A', '17:45:12', '-28:48.3', '0.1');
}

function load() {
    loadExampleList();
}

$(document).ready( function () {
    $('a#exampleInputCoordinates').click( function () {
        $('div#exampleInputs').slideToggle();
    });
    $('div#exampleInputs').hide();
    $('div#getVotableFromUrl').hide();
});

var aoDisplayWindows = [];

/* Display Crossmatch results in new windows.  The new window will invoke
 * VOView to display the VOTables returned by the SCS services.  SCS query
 * parameters are passed to the new window via the URL (i.e. HTTP GET 
 * parameters).  The new window objects are added to the aoDisplayWindows
 * array.
 */
function startVoview(element) 
{
    var displayUrlBase = 'display.html?'
    var displayUrl;

    // If input is an explicit URL...
    if ( element && element.id == "urlSubmit" ) 
    {
        var url = getUrlInput();
        if (! url ) 
        { 
            alert("A URL to a VO table must be supplied."); 
            return; 
        }
        displayUrl = displayUrlBase +
            'URL=' + encodeURIComponent( url );
        // Open display in new window and add window object to array
        aoDisplayWindows.push( window.open(displayUrl) );
        return;
    }
    else // if input is SCS query...
    {
        var asScs = getScsServices();
        for (var i=0; i < asScs.length; i++) 
        {   
            // URL for invoking Crossmatch to display the SCS service response
            displayUrl = displayUrlBase +
                'SCS=' + encodeURIComponent( asScs[i] ) + 
                '&RA=' + encodeURIComponent( getRaInput() ) + 
                '&DEC=' + encodeURIComponent( getDecInput() ) + 
                '&SR=' + encodeURIComponent( getRadiusInput() );
            // Open display in new window and add window object to array
            aoDisplayWindows.push( window.open(displayUrl) );
        }
        return;
    }
}

/* Return the values of the options selected in the SCS select element.
 * Multiple options can be selected simultaneously.
 */
function getScsServices()
{
    var asSelected = [];
    nSelect = document.getElementById("scs");
    for (var i = 0; i < nSelect.options.length; i++) 
    {
        if (nSelect.options[i].selected) 
        {
            asSelected.push(nSelect.options[i].value);
        }
    }
    return asSelected;
}

/* Source look up functions.
 *
 * Allow user to look-up coordinates by source name using SIMBAD and NED.
 * 1) Accept source name input in a text field.
 * 2) User clicks "SIMBAD/NED look-up" button.
 * 3) JS builds SIMBAD/NED query URL.
 * 4) Open URL in new window.
 * 
 * I decided not to try and fill in the RA and Dec fields automatically 
 * for several reasons: (1) this would require error handling, in case the
 * user enters an improper source name; (2) browser security models do not 
 * allow cross-domain AJAX queries.
 *
 */

/* Open a SIMBAD query in a new window. */
function simbadQuery() {
    var simbadUrl = "http://simbad.harvard.edu/simbad/sim-id?Ident="
    sourceName = $('span#queryIdentifier')[0].name;
    var url = simbadUrl + encodeURI(sourceName);
    window.open(url);
}

/* Open a NED query in a new window. */
function nedQuery() {
    var nedUrl = "http://nedwww.ipac.caltech.edu/cgi-bin/nph-objsearch?extend=no&objname="
    sourceName = $('span#queryIdentifier')[0].name;
    var url = nedUrl + encodeURI(sourceName);
    window.open(url);
}

/* Get the source name from the text box and verify it's not empty. */
function lookUp() {
    var sourceName = getLookUpInput();
    if ( sourceName == "" ) {
        document.getElementById('lookUp_feedback').innerHTML = "<span class='warn'> Source identifier is required for look up. </span>";
        return;
    } else {
        document.getElementById('lookUp_feedback').innerHTML = "";
    }
    return sourceName;
}
