package crossmatch;

import org.apache.log4j.Logger;
import dalclient.*;
import java.io.InputStream;
import java.io.File;

/** Query the Simple Cone Search service, verify the response table structure,
  * parse the response table to acquire useful data, write the VO table to disk,
  * and hold the response, and provide for access to the response and its 
  * constituents.
  *
  */
public class SCSQuery {
    
    private Logger logger;
    
    private String serviceUrl;
    double searchRA;
    double searchDec;
    double searchSR;
    ConeQuery query;
    private InputStream responseStream;
    boolean originalVOTable;
        
    /** Constructor: 
      *   1) Execute query, 
      *   2) Generate VOTable object
      */
    public SCSQuery(String qUrl, double qRa, double qDec, double qSr) throws Exception {
        logger = Logger.getLogger(SCSQuery.class);
        // Initialize inputs
        serviceUrl = qUrl;
        searchRA = qRa; searchDec = qDec; searchSR = qSr;
        // Query the service.
        ConeConnection cone = new ConeConnection(serviceUrl);
        query = cone.getConeQuery(searchRA, searchDec, searchSR);
        logger.info("Sending Query: \n" + getQueryString());
        responseStream = query.executeRaw();
    }
        
    /** Get the SCS service base URL. */
    public String getServiceUrl() { return serviceUrl; }

    /** Get the HTTP URL */
    public String getQueryString() { return query.getQueryString(0); }    

    /** Get Response */
    public InputStream responseStream() { return responseStream; }
    
}