package crossmatch;

import org.apache.log4j.Logger;
import javax.servlet.http.*;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.UnknownHostException;
import java.net.URL;
import jhc.ReadString;
import java.io.BufferedReader;
import java.io.PrintWriter;
import java.io.InputStreamReader;

/** Perform a Sesame query */
class Sesame {

    private Logger logger;
    private static final String sesameUrlBase = 
        "http://cdsweb.u-strasbg.fr/cgi-bin/nph-sesame/";
    // Return info from Simbad, NED, and Vizier when available
    // See http://cdsweb.u-strasbg.fr/doc/sesame.htx for info about options
    private static final String sesameUrlOptions = "-ox/SNVA?";
    
    private String content;
    private String contentType;
    private int contentLength;

    /** Constructor 
      * Query Sesame service using the given identifier.  Set the content and content 
      * type of the response.
      * I SHOULD LOOK MORE CLOSELY AT THE ERROR HANDELING HERE TO SEE HOW TO SET THE 
      * FIELDS WHEN AN ERROR OCCURS.
      */
    public Sesame(String identifier) throws MalformedURLException {
        logger = Logger.getLogger(Sesame.class);
        logger.info("Performing sesame lookup for identifier " + identifier);
        identifier = identifier.replace(' ', '+');
        String urlString = sesameUrlBase + sesameUrlOptions + identifier;
        logger.debug("Creating URL: " + urlString);
        URL url = new URL(urlString);       
        try {
            // Query service
            HttpURLConnection connect = (HttpURLConnection)url.openConnection();
            logger.info("Response code: " + connect.getResponseCode());
            logger.info("Response message: " + connect.getResponseMessage());
            contentType = connect.getContentType();
            if (! contentType.equals("text/xml") ) 
            {
                throw new Exception("Unexpected response content type: " + connect.getContentType());
            }
            contentLength = connect.getContentLength();
            logger.debug("Response content length = " + contentLength + " (-1 = unknown)");
            BufferedReader br = new BufferedReader( new InputStreamReader( connect.getInputStream() ) );
            int i;
            while ( ( i = br.read() ) != -1 ) {
                if (content == null) { content = ""; }
                content = content + (char)i;
            }
        }
        catch (UnknownHostException e) {
            logger.error("UnknownHostException: " + e);
        }
        catch (Exception e) {
            logger.error(e.getMessage());
        }
    }

    /** Fill an HttpServletResponse, in order to pass the content of the
      * Sesame response back to the client.  This will be used for an AJAX Sesame Query.
      */
    public void fillResponse(HttpServletResponse response) {
        response.setContentType(contentType);
        response.setBufferSize(contentLength);
        try {
            PrintWriter pw = response.getWriter();
            pw.print(content);
            pw.close();
        }
        catch (Exception e) {
            logger.error(e.getMessage());
        }
    }
    
    /** Test sesame lookup from terminal. */
    public static void main(String[] args) throws MalformedURLException {
        Logger logger = Logger.getLogger(Sesame.class);
        ReadString rs = new ReadString();
        String identifier = rs.readString("Input identifier:");
        Sesame sesame = new Sesame(identifier);
        logger.info(sesame.getContent());
    }
    
    /** Return content */
    public String getContent() { return content; }
    /** Return content type */
    public String getContentType() { return contentType; }
}

