package crossmatch;

import org.apache.log4j.Logger;
import javax.servlet.http.*;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.UnknownHostException;
import java.net.URL;
import java.io.BufferedReader;
import java.io.PrintWriter;
import java.io.InputStreamReader;

/** Submit a request for a VO table document given an explicit URL. 
  * This class is modeled after SCSQuery, and in fact this class could
  * probably be a parent of SCSQuery!  
  *
  * MAKE THIS A PARENT OF SCSQUERY?
  */
class UrlRequest {

    private Logger logger;

    private String content;
    private String contentType;
    private int contentLength;

    /** Sends an HTTP request to a specified URL and stores the response
      * in content.  
      */
    public UrlRequest(String urlString) throws MalformedURLException {
        logger = Logger.getLogger(UrlRequest.class);
        URL url = new URL(urlString);
        try {
            // Make request
            HttpURLConnection connect = (HttpURLConnection)url.openConnection();
            logger.info("Response code: " + connect.getResponseCode());
            logger.info("Response message: " + connect.getResponseMessage());
            contentType = connect.getContentType();
            // if (! contentType.equals("text/xml") ) 
            // {
            //     throw new Exception("Unexpected response content type: " + 
            //         connect.getContentType());
            // }
            contentLength = connect.getContentLength();
            logger.debug("Response content length = " + contentLength + 
                " (-1 = unknown)");
            BufferedReader br = new BufferedReader( 
                new InputStreamReader( connect.getInputStream() ) );
            int i;
            while ( ( i = br.read() ) != -1 ) {
                if (content == null) { content = ""; }
                content = content + (char)i;
            }
        }
        catch (UnknownHostException e) {
            logger.error("UnknownHostException: " + e);
        }
        catch (Exception e) {
            logger.error(e.getMessage());
        }
    }

    /** Fill an HttpServletResponse, in order to pass the content of the
      * response back to the client.  
      */
    public void fillResponse(HttpServletResponse response) {
        response.setContentType(contentType);
        response.setBufferSize(contentLength);
        try {
            PrintWriter pw = response.getWriter();
            pw.print(content);
            pw.close();
        }
        catch (Exception e) {
            logger.error(e.getMessage());
        }
    }

    /** Return content */
    public String getContent() { return content; }
    /** Return content type */
    public String getContentType() { return contentType; }

}
