package jhc;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.File;
import java.io.IOException;
import java.io.FileNotFoundException;
import org.apache.log4j.Logger;
import java.util.HashMap;

/** This class is a utility for reading input files.
  **/
public class InputFile {

    private Logger logger;
    
    private HashMap<String,String> hm;
    private BufferedReader input;
    
    public InputFile(String filename) throws FileNotFoundException, IOException {
        logger = Logger.getLogger(InputFile.class);
        try {
            input = new BufferedReader(new FileReader(new File(filename)));
            parseKeyVal();
            input.close();
        }
        catch (FileNotFoundException ex) {
            logger.fatal("Designated input file ( " + filename + " ) not found. Aborting.");
            throw(ex);
        }
        catch (IOException ex) {
            logger.error("Exception thrown while trying to close BufferedReader for input file " + filename);
            throw(ex);
        }
    }
    
    /** Parse a file of key-value pairs, returning them in a HashMap with keys
      * and values stored as Strings.
      *
      * Rules:
      * 1. Blank lines are ignored
      * 2. For each line, ignore everything after a "#" character.
      * 3. All test inputs are written as key-value pairs
      *    separated by a colon and 1 or more whitespace
      *    characters.
      * 4. Keys must be unique.  Values need not be unique.
      **/
    private void parseKeyVal() {
        try {
            hm = new HashMap<String,String>();
            /* readLine is a bit quirky :
             *   it returns the content of a line MINUS the newline.
             *   it returns null only for the END of the stream.
             *   it returns an empty String if two newlines appear in a row.
             */
            String line = null;
            while (( line = input.readLine()) != null ) { // while not end of stream
                if (line.length() > 1) {
                    String live = line.split("#",2)[0]; // live holds everything before the "#"
                    if (live.length() > 0) { 
                        String[] keyVal = live.split(":\\s+"); // split live at ":[whitespace]"
                        logger.debug("Found Key: " + keyVal[0] + ", Value: " + keyVal[1]);
                        
                        // Remove leading and trailing whitespace from keys and values.
                        for(int i=0; i<keyVal.length; i++) {
                            keyVal[i] = keyVal[i].trim();
                        }
                        
                        // Add the key and value to the HashMap, hm.
                        hm.put(keyVal[0], keyVal[1]);
                    }    
                }
            } // end while
        } // end try
        catch (IOException ex){
            ex.printStackTrace();
            // For servlet operation, store error message and throw another exception;
            // the second exception should cause LoadTest to skip starting the test
            // and instead return an HTML page stating what error occured.
        }
        logger.debug(hm.toString());
    } // end parseKeyVal(..)
    
    public HashMap<String,String> hm() { return hm; }
    
} // end class