/* Constructor for RowGroups objects.
 * PARAMETERS: iTableIndex = index to the oTable element (and $('.tableSet') element) to be grouped. 
 * The constructor returns without finishing if the table description is not defined in function fnDefineGroupColumnLabels.
 */
function RowGroups( iTableIndex ) {

    var that = this; // make object accessible to methods

    /* PUBLIC METHODS */
    
    // Return A COPY of aaiSummaryToRow (summary rows --> data rows)
    this.fnGetSummaryToRow = function() 
        { return $.extend(true, [], aaiSummaryToRow); }
    
    // Return A COPY of aiRowToSummary (data rows --> summary rows)
    this.fnGetRowToSummary = function() 
        { return $.extend(true, [], aiRowToSummary); }
        
    this.fnIsGroupable = function () { return bGroupable; }

    /* Initialize Row Groups */
    this.fnInitGroups = function () {
        fnSetGroupColumnIndices();
        fnGatherGroupRows(); // Form the row groups.
        // If no groups have been created, collapse the row-group-button column and return...
        if (aaiSummaryToRow.length == 0) 
        { 
            fnReturnEarly(); 
        } 
        else // else, initialize the display
        { 
            $(nDomTable).find('col.rowGroupButton')[0].style.visibility="visible";
            fnInitDisplayedRows(); 
        }
        return;
    }

    /* Expand group: remove summary row and replace it with all data rows in the group.  iDataIndex = index of the group summary row in aoData. */
    this.fnExpandGroup = function ( iDataIndex ) {
        var iDispMastIndex = $.inArray(iDataIndex, aiDisplayMaster);
        var aiGroupRowIndices = aoRowGroups[iTableIndex].fnGetSummaryToRow()[iDataIndex];
        if (iDispMastIndex < 0) { 
            alert("Error: Attempted to remove row not present in aiDisplayMaster.");
            return;
        };
        /* Make changes to aiDisplayMaster */
        aiDisplayMaster.splice(iDispMastIndex, 1); // remove summary
        //alert("iTableIndex = " + iTableIndex + "\n iDataIndex = " + iDataIndex + "\n iDispMastIndex = " + iDispMastIndex + "\n aiGroupRowIndices = " + aiGroupRowIndices);
        // Insert Group rows
        for (var i = 0; i < aiGroupRowIndices.length; i++) {
            aiDisplayMaster.splice(iDispMastIndex, 0, aiGroupRowIndices[i]);
        }
        /* Make changes to aiDisplay, if necessary */
        var iDispIndex = $.inArray(iDataIndex, oSettings.aiDisplay);
        if (iDispIndex > -1) {
            oSettings.aiDisplay.splice(iDispIndex, 1); // remove summary
            // Insert group rows
            for (var i = 0; i < aiGroupRowIndices.length; i++) {
                oSettings.aiDisplay.splice(iDispIndex, 0, aiGroupRowIndices[i]);
            }
        }
    }

    /* Collapse a group: remove group data rows and replace with summary row.  iDataIndex = index of the selected row in aoData. */
    this.fnCollapseGroup = function ( iDataIndex ) {
        var iSummaryIndex = that.fnGetRowToSummary()[iDataIndex];
        var aiGroupRowIndices = that.fnGetSummaryToRow()[iSummaryIndex];
        // Find where in the aiDisplay array the clicked element resides; track this position for summary row insertion
        var iFirstDispIndex = $.inArray(iDataIndex,oSettings.aiDisplay);
        // Remove group data rows
        for (var j = 0; j < aiGroupRowIndices.length; j++) {
            var iDispMastIndex = $.inArray(aiGroupRowIndices[j],aiDisplayMaster);
            if (iDispMastIndex > -1) {
                aiDisplayMaster.splice(iDispMastIndex,1);
            } else { 
                alert("Error: Attempted to remove a row in this group, but row is not displayed in table.");
            }
            var iDispIndex = $.inArray(aiGroupRowIndices[j],oSettings.aiDisplay);
            if (iDispIndex > -1) {
                /* Track the display position of the clicked row for proper 
                   summary insertion. */
                if ( iDispIndex < iFirstDispIndex ) { iFirstDispIndex--; }
                oSettings.aiDisplay.splice(iDispIndex,1);
            } 
        }
        // Add group summary row
        aiDisplayMaster.push(iSummaryIndex);
        if (iFirstDispIndex > -1) 
        {
            oSettings.aiDisplay.splice(iFirstDispIndex,0,iSummaryIndex);
        } 
        else 
        { 
            alert("Error: Clicked row is not present in aiDisplay array.");
        }
    }

    /* Expand all groups on all pages of the current table. */
    this.fnExpandAllGroups = function () {
        var aiSTR = that.fnGetSummaryToRow();
        var i; // loop index
        for (i = 0; i < aiDisplayMaster.length; i++) {
            if ( aiSTR[ aiDisplayMaster[i] ] ) { // if row is in a group
                that.fnExpandGroup( aiDisplayMaster[i] );
            }
        }
        oDataTable.fnDraw();
    }
    
    /* Collapse all groups on all pages of the current table. oDataTable = the applicable DataTable object. */
    this.fnCollapseAllGroups = function () {
        var i; // loop index
        for (i = 0; i < aiDisplayMaster.length; /* increment in block */ ) 
        {
            // If this row is in a group, collapse group.  Do not increment i.
            if ( aiRowToSummary[ aiDisplayMaster[i] ] ) 
            { 
                that.fnCollapseGroup( aiDisplayMaster[i] );    
            } 
            else // If this row is not in a group, increment i.
            {
                i++;
            }
        }
        oDataTable.fnDraw();
    }

    /* Redraw that table without chaning the current page.  This simply requires that filtering and sorting be temporarily turned off during the draw process. */
    this.fnStaticDraw = function () {
/*        
        // Allan Jardine's method #1
        // Additional changes must be made to DataTables code.  (See http://datatables.net/forums/comments.php?DiscussionID=573 ).
        bResetDisplay = false; // override default behaviour
        oDataTable.fnDraw();
        bResetDisplay = true; // restore default behaviour
*/    
        // Allan Jardine's method #2
        oSettings.oApi._fnCalculateEnd( oSettings ); 
        oSettings.oApi._fnDraw( oSettings ); 
/*     
        // Jared's method (Allan says this may not work in future version of DataTables.  See http://datatables.net/forums/comments.php?DiscussionID=607 ).
        oSettings.oFeatures.bFilter = false;
        oSettings.oFeatures.bSort = false;
        oDataTable.fnDraw();
        oSettings.oFeatures.bFilter = true;
        oSettings.oFeatures.bSort = true;
*/      
/*  
        // Non-static draw
        oDataTable.fnDraw();
*/
    }

    /* PRIVATE FIELDS */
    
    // Constructor input parameter
    var iTableIndex = iTableIndex;
    
    var bGroupable; // Table can be grouped.
    var oDataTable = oTable[iTableIndex]; // applicable DataTable object
    var oSettings = oDataTable.fnSettings();
    var aiDisplayMaster = oSettings.aiDisplayMaster;
    
    var nDomTable = $('.tableSet')[iTableIndex]; // applicable DOM table object
    
    // Array that stores arrays of indices for each group summary row created; array elements are defined only for the aoData indices where summary rows are added.
    var aaiSummaryToRow = []; 
    
    // Array that stores indices of the aoData summary row for each data row put into a group.  If row aoData[5] is put into a group that has a summary row at aoData[89], then aiRowToSummary[5] = 89. A list of all rows in the group can be obtained by using aaiSummaryToRow, as in aaiSummaryToRow[ aiRowToSummary[5] ].
    var aiRowToSummary = new Array(oDataTable.fnGetData().length);
    
    // Associative array that holds the lables of columuns used for row grouping; array is indexed by the table description string.
    var asGroupColumnLabels = fnDefineGroupColumnLabels();
    
    // Array of indices to the columns that will be used for row matching.
    var aiGroupColumnIndices = [];
    
    // Table description
    var tableDescription = $(nDomTable).find('.tableDescription').html();       

    // Image to click for group expansion
    var sExpandButtonInnerHTML = "<img src='js/dataTables-1.5/examples/examples_support/details_open.png'>";
    // Image to click for group collapse
    var sCollapseButtonInnerHTML = "<img src='js/dataTables-1.5/examples/examples_support/details_close.png'>";

    /* PRIVATE METHODS */
    
    /* No groups were created.  Collapse the row-group-button column and return. */
    function fnReturnEarly() {
        $(nDomTable).find('col.rowGroupButton')[0].style.visibility="collapse";
        return;
    }
    
    /* Initialize the table so only data rows are visible.  That is, remove all summary rows from arrays of displayed rows. When groups are created, both the data rows and summary rows are visible.  This function removes summary rows from the display. */
    function fnInitDisplayedRows() {
        var i; // loop index
        for (i = 0; i< aaiSummaryToRow.length; i++) {
            if ( ( typeof aaiSummaryToRow[i] ) != "undefined" ) {
                var iDispMastIndex = $.inArray(i, aiDisplayMaster);
                var iDispIndex = $.inArray(i,oSettings.aiDisplay);                
                if ( iDispMastIndex >= 0 ) 
                { 
                    aiDisplayMaster.splice(iDispMastIndex,1);
                } else 
                { 
                    alert("Summary row not listed in aiDisplayMaster!"); 
                }
                if ( iDispIndex >= 0 )
                {
                    oSettings.aiDisplay.splice(iDispIndex,1);
                }
            }
        }
    }
    
    /* Return an array of indices to the columns that will be used for
     * matching during the row grouping process.
     */
    function fnSetGroupColumnIndices() {
        // Get the column label row from DOM
        var aoColumnLabelRow = 
                $(nDomTable).find('tr.columnLabel')[0].cells;
        //alert('columnLabel cell [0] = ' + aoColumnLabelRow[0]);
        //alert('columnLabel cell [0] innerHTML = ' + aoColumnLabelRow[0].innerHTML);
        var asColumnLabels = []; // Holds all column labels
        var i; // loop variable
        for ( i=0; i < aoColumnLabelRow.length; i++ ) {
            var oCell = aoColumnLabelRow[i];
            asColumnLabels.push(oCell.innerHTML);
        }
        /* Iterate though the pre-defined list of columns that will be 
         * grouped on, creating an array of column indices. */
        for (i = 0; i < asGroupColumnLabels[tableDescription].length; i++ ) {
            var label = asGroupColumnLabels[tableDescription][i]
            var index = jQuery.inArray(label, asColumnLabels);
            if (index != -1) { aiGroupColumnIndices.push(index); }
        }
    }

    /* Find matching rows and create row groups. */
    function fnGatherGroupRows() {
        /* Get the table data rows and create an array of indices to the table rows. */
        var aasTbodyData = oDataTable.fnGetData(); // Table data
        //var aiSortedIndices = fnGetSortIndices();
        
        var aiRowIndices = []; // array of table row indices (for grouping algorithm)
        var i; // loop index
        for (i = 0; i < aasTbodyData.length; i++) { aiRowIndices[i] = i; }
        
        // Iterate through the ungrouped rows of tbody...
        while (aiRowIndices.length > 1) 
        {
            var rowIndexA = aiRowIndices[0];
            var aiGroupRowIndices = []; // hold indices to matched rows
            var iSummaryIndex = null; // index of summary row in aoData
            var bMatch = true; // The rows were matched.

            for ( i = 1; i < aiRowIndices.length; i++ ) 
            {
                bMatch = fnCompareRows( aasTbodyData[ rowIndexA ], 
                                        aasTbodyData[ aiRowIndices[i] ] );
                if ( bMatch )
                {
                    // Add row to group
                    aiGroupRowIndices.push( aiRowIndices[i] ); 
                }
            }
            // Add rowIndexA to group
            aiGroupRowIndices.push( rowIndexA ); 

            if ( aiGroupRowIndices.length > 1 ) 
            {
                // create a summary row
                iSummaryIndex = fnCreateGroup(aiGroupRowIndices, aasTbodyData[rowIndexA]); 
                aaiSummaryToRow[iSummaryIndex] = aiGroupRowIndices;
                for ( i = 0; i < aiGroupRowIndices.length; i++ ) 
                {
                    // Associate summary row index with each row in group
                    aiRowToSummary[ aiGroupRowIndices[i] ] = iSummaryIndex;
                    /* Add the image via XSL
                    // Add collapse button to each row in group
                    oDataTable.fnUpdate(sCollapseButtonInnerHTML,aiGroupRowIndices[i],0,false);
                     */
                    // Define click event for row-group collapse image
                    $('td img', oDataTable.fnGetNodes(aiGroupRowIndices[i])).click( fnCollapseGroupWButton );

                }
            }
            else // if only 1 row in group, remove collapse button
            {
                oDataTable.fnUpdate('',aiGroupRowIndices[0],0,false);
                //alert("Removing image for row " + aiGroupRowIndices[0]);
            }

            // Remove all rows in group from aiRowIndices
            for (i = 0; i < aiGroupRowIndices.length; i++) 
            {
                var index = $.inArray(aiGroupRowIndices[i], aiRowIndices);
                aiRowIndices.splice(index , 1);
            }
            
        }
        
        /* Return the value of the index-th element of the sorted data array.
         */
        function fnGetSortData( index )
        {
            return aasTbodyData[ aiSortedIndices[ index ] ];
        }
        
        /* Compare two rows of table data to determine if they have equal values in grouping columns.  If values in grouping columns are all equal, return true; else, return false. */
        function fnCompareRows( asDataRow_A, asDataRow_B) 
        {
            var i; // loop variable
            for ( i=0; i < aiGroupColumnIndices.length; i++ ) 
            {
                if ( asDataRow_A[ aiGroupColumnIndices[i] ] != 
                     asDataRow_B[ aiGroupColumnIndices[i] ] ) { return false; }
            }
            return true;
        }

        /* Create a "group" of data rows given the row indices.  1) Create a summary row that will be added to DataTables array aoData.  2) Add an object to the RowGroupInfo array; the object will contain an index pointing to the summary row in aoData and an array of indices pointing to the rows contained in this group.
           PARAMETERS: 
           aiGroupRowIndices = array of indices to rows in the group;
           asAnyDataRow = any row in the group (array of strings) */ 
        function fnCreateGroup( aiGroupRowIndices, asAnyDataRow ) 
        {
            //alert("asAnyDataRow = " + asAnyDataRow);
            // Create summary row with same number of columns
            var asSummaryRow = new Array(asAnyDataRow.length);
            var i; // array index
            
            // First cell is the group-expansion button
            asSummaryRow[0] = sExpandButtonInnerHTML;
            
            // Other cell values default to empty strings
            for (i = 1; i < asSummaryRow.length; i++) { asSummaryRow[i] = ""; }
            for (i = 0; i < aiGroupColumnIndices.length; i++) 
            {
                var index = aiGroupColumnIndices[i];
                asSummaryRow[index] = asAnyDataRow[index];
            }
            // Add summary row to DT's aoData and get return index
            var iSummaryIndex = oDataTable.fnAddData(asSummaryRow, false)[0];
            oSettings.aoData[iSummaryIndex].nTr.className="rowGroupSummary";
            
            // Define click event for row-group expansion image.
            $('td img',oDataTable.fnGetNodes(iSummaryIndex)).click( fnExpandGroupWButton );
            //alert(iSummaryIndex);
            return iSummaryIndex;
        }
        
        /* Return indices to the rows of aoData, sorted by the columns used for row grouping.  
           Finally, if possible, return table display to original condition.
         */
        function fnGetSortIndices() 
        {
            var aaSortArray = [];
            var aiSortedIndices; // This might be sorted in the containing function?
            var i; // loop index
            
            // REMOVING FILTERS IS NOT NECESSARY IF GROUPS ARE FORMED AT STARTUP
            // DataTable.fnFilter(''); // remove any filters            
            
            for( i=0; i < aiGroupColumnIndices.length; i++ )
            {
                aaSortArray[i] = [ aiGroupColumnIndices[i], 'asc' ];
            }
            oDataTable.fnSort( aaSortArray ); // sort table
            aiSortedIndices = oSettings.aiDisplay; // SHOULD REF THIS ARRAY OR BUILD A NEW ARRAY?
            
            // Return display to original condition: filtering, sortings, pagination
            // Refreshing filtering may be as easy as using jQuery to fake a change to the filter text boxes.
            // I'll have to figure out how to save the original sort state; what I need is something like the input array given to fnSort, but prior to my changing the display in this function.
            // I've reasearched about pagination before.  This should be doable.
            return aiSortedIndices;
        }
        
    } // end fnGatherGroupRows
    
    /* Expand a single group */
    function fnExpandGroupWButton() {
        // Here this refers to the <img> element that holds a reference to this function.
        var nTr = this.parentNode.parentNode;
        var iDataIndex = oDataTable.fnGetPosition( nTr );
        that.fnExpandGroup( iDataIndex );
        that.fnStaticDraw();
    }

    /* Collapse a single group using a button. A reference to this function should be assigned to the <img> elements in the data rows. */
    function fnCollapseGroupWButton() {
        //alert("In funCollapseGroupWButton, this = " + this);
        var nTr = this.parentNode.parentNode;
        var iDataIndex = oDataTable.fnGetPosition(nTr);
//        var iStart = oSettings._iDisplayStart;
//        alert("iStart = " + iStart);
        that.fnCollapseGroup( iDataIndex );
//        oSettings.iDisplayStart = iStart;
        that.fnStaticDraw();
    }
        
    /* Define group column labels for select table descriptions.  These columns will be used to create row groups. */
    function fnDefineGroupColumnLabels() {
        var as = [];
        as["VLA Observation Summary Catalog"] = [ "project_code", "source_id", "if_band", "telescope", "telescope_config", "ra2000", "dec2000" ];
        // Add new table group column labels here...
        return as;
    }

    // Check to see if the tableDescription is a property of (associative array) asGroupColumnLabels.  If not, then rows cannot be grouped.
    if (! asGroupColumnLabels.hasOwnProperty(tableDescription)) 
    { 
        bGroupable = false;
    }
    else
    {
        bGroupable = true;
    }
    return;
        
} // end fnRowGroups constructor
