/* Main JavaScript code for the NRAO Cross-identification Application.
 */
 
var asInitVals = new Array(); 
var oTable = new Array(2); // Array of DataTables objects 
var aoRowGroups = [null, null]; // Array of RowGroups objects
var bResetDisplay = true; // work around to avoid DataTables resetting display

/* Initialize DataTables jQuery plugin */
$(document).ready(function() {
    
    // Hide inputs panel
    $('a#viewInputs').click( function () {
        $('div#inputs').slideToggle();
    });
    $('div#inputs').hide();
    
    // Initialize DataTables for both data tables in the DOM
    for (var tableIndex = 0; tableIndex < 2; tableIndex++) 
    {
        aoColumnsOptArr = [] //0, {"sWidth":"20%"}, // row-group button column
                            //0, {"bSortable": false} ]; // button column
        var oThisTable = $(".responseTable")[tableIndex];
        // Create DataTables objects
        oTable[tableIndex] = $(oThisTable).dataTable( 
        {
            "aoColumns": setColumnOptions( countTableCols(oThisTable),
                                           aoColumnsOptArr ),
            "aaSorting": [[1, 'asc']],
            "bProcessing": true,
            "sPaginationType": "full_numbers",
            "fnDrawCallBack": function() {alert('DataTables has redrawn the table');},
            "bAutoWidth": false
            // "bSortClasses": false // make DT run faster
            //"bStateSave": true
        });
    }

    //Initialize table controls
    $('.tableControlSubPanel').hide();
    $('.tableControlPanel').each( function () 
    {
        // Activate column-toggle subpanel
        $(this).find('.columnToggleButton').click( function () {
            $(this).parent().find('.columnToggles').slideToggle();
        });
        
        // Activate column-header subpanel
        $(this).find('.columnHeaderToggleButton').click( function () {
            $(this).parent().find('.columnHeaders').slideToggle();
        });
        // Activate column-header UCDs toggle
        $(this).find('.columnUcdsToggleButton').click( function() {
            $(this).closest('.tableSet').find('.columnUcds').toggle();
            $(this).toggleClass("tableEventButton_hidden");
        });
        // Activate column-header units toggle
        $(this).find('.columnUnitsToggleButton').click( function() {
            $(this).closest('.tableSet').find('.columnUnits').toggle();
            $(this).toggleClass("tableEventButton_hidden");
        });
        
        // Activate row groups subpanel
        tableIndex = fnGetTableIndex( this );
        // Construct the row grouping object
        aoRowGroups[tableIndex] = new RowGroups( tableIndex );
        // If this table can be grouped...
        if ( aoRowGroups[tableIndex].fnIsGroupable() ) 
        { 
            $(this).find('.rowGroupToggleButton').click( function() {
                $(this).parent().find('.rowGroupSubPanel').slideToggle();
            });
            // Activate group-activation button
            $(this).find('.activateRowGroups').click( function () 
            {
                oTS = $(this).closest('.tableSet');
                span = oTS.find('span.rowGroupsActivationStatus')
                // Initialize groups 
                aoRowGroups[ fnGetTableIndex( this ) ].fnInitGroups();
                aoRowGroups[ fnGetTableIndex( this ) ].fnCollapseAllGroups();
                // Remove button; replace with text
                span.attr( "style", "font-style:normal; font-weight:bold;" );
                span.text("Row grouping is active");
                // Unhide buttons
                oTS.find('.expandAllGroupsButton').removeClass('tableEventButton_hidden');
                oTS.find('.collapseAllGroupsButton').removeClass('tableEventButton_hidden');
            });
            // Activate expand all row groups
            $(this).find('.expandAllGroupsButton').click( aoRowGroups[ fnGetTableIndex( this ) ].fnExpandAllGroups );
            // Activate collapse all row groups
            $(this).find('.collapseAllGroupsButton').click( aoRowGroups[ fnGetTableIndex( this ) ].fnCollapseAllGroups );
        } 
        else // Else, if it cannot be grouped...
        {
            aoRowGroups[tableIndex] = null;
            // Shade the row group button
            $(this).find('.rowGroupToggleButton').addClass("tableEventButton_hidden");
        }

    });
    
    perColumnFilters();
    
    fnInitHideColumns();
    
});

/* Initially hide columns for specified tables. The tables with initially hidden columns, and the columns that should be hidden, are stored in the associative array 'as'. */
function fnInitHideColumns() {
    // Array of columns to initially hide
    var as = [];
    as["VLA Observation Summary Catalog"] = ["sub_chan_id", "n_images", "file_set_id", "arch_file_id", "obssummary_uid"];
    var asColumns; // columns to hide
    var i = 0; // loop index
    // add entries for other tables here...
    $('.tableSet').each( function () {
        sTableDescription = $(this).find('.tableDescription').html();
        if ( as.hasOwnProperty( sTableDescription ) ) {
            anCTs = $(this).find('.columnToggles');
            asColumns = as[ sTableDescription ];
            for (i = 0; i < asColumns.length; i++) {
                $(anCTs).find('a:contains(' + asColumns[i] + ')').click();
            }
        }
    });
}

/* Given some DOM node, find the index of the ancestor div.tableSet node containing the DOM node.  This index will be 0 or 1 and will be the index corresponding to the DataTables object for this table. */
function fnGetTableIndex( node ) {
    var jAncestor = $(node).closest('.tableSet');
    var jTableSets = $('.tableSet');
    var iIndex = jTableSets.index(jAncestor);
    return iIndex;
}

/* Hide and unhide table columns. 
 * Use the DataTables function fnSetColumnVis to show/hide columns because it works in both Firefox and Safari.  Using the CSS visibility property (setting to "collapse" to hide) only works in Firefox.  When hidden, the button corresponding to this column in the table control panel should be shaded.  
 *
 * This function is invoked by buttons in the table control panel; see XSL for function call. 
 *
 * Note that prepended columns cannot be hidden and therefore do not show up in the list of column-toggle buttons.  The toggle button index is equal to the table column index Plus the number of prepended columns; subtract 1 because the XPath position() function starts counting at 1, JS counts from zero.
 */
function fnShowHide( iCol, clickedElement )
{   
    // Convert from button to table column index
    iCol = iCol - 1 + fnGetNumPrependCols( clickedElement );
    var tableNum = fnGetTableIndex( clickedElement );
    var bVis = oTable[ tableNum ].fnSettings().aoColumns[iCol].bVisible;
    // Show/Hide column using DataTables function
    oTable[ tableNum ].fnSetColumnVis( iCol, bVis ? false : true );
    // When hidden, shade corresponding button in control panel
    clickedElement.style.color = ( bVis ) ? "DarkGray" : "";
}

/* Add individual column filters to the table. 
 *   This is a feature provided by DataTables.  The code below is a 
 *   modification of code from the DataTables web site.
 */
function perColumnFilters() {
    // Individual column filter
	$("tfoot input").keyup( function () {
        var nTable = $(this).closest("table")[0];
        var iTableIndex = fnGetTableIndex(nTable);
        //alert("table index = " + iTableIndex);
        // Get the oTable column corresponding to the active input box
        var nTh = $(this).closest("th")[0];
        var iThIndex = $(nTable).find("tfoot tr th").index(nTh);
        //alert("(tfoot) th index = " + iThIndex);
        
// fnGetPosition seems to be returning the indices of the displayed data,
// not indices of the original aoData.  This surprises me.  Do I have the 
// DOM element selection correct?  
        var aiPosition = oTable[iTableIndex].fnGetPosition( $(nTable).find("tbody tr td:eq(" + iThIndex + ")")[0] );
        var iCol = aiPosition[1]; // active oTable column
        
        //alert("Index of th element = " + iThIndex + "\n oTable column index = " + iCol);
        
        oTable[iTableIndex].fnFilter( this.value, iCol);
	} );
    
    // Set default (background) value for per-column search boxes
	$("tfoot input").each( function (i) {
		asInitVals[i] = this.value;
	} );
	
    // When per-column search box is made empty, deactivate it
	$("tfoot input").focus( function () {
		if ( this.className == "search_init" )
		{
			this.className = "";
			this.value = "";
		}
	} );
	
    // When per-column search box is in use, activate it
	$("tfoot input").blur( function (i) {
		if ( this.value == "" )
		{
			this.className = "search_init";
            this.value = asInitVals[$("tfoot input").index(this)];
		}
	} );
}

/* Get the index of the dataTable object corresponding to the table whose
 * ID is equal to tableId.
 */
function getDataTableIndex(tableId) 
{
    std = "responseTable1"
    if ( tableId == std || tableId == "#" + std) 
        { return 0; }
    else 
        { return 1; }
}

/* Count the number of columns in the table
 *   PARAMETERS:
 *     elementId = ID of the table element of interest
 */
function countTableCols(oThisTable) 
{
    // return window.document.getElementById(elementId).rows[0].cells.length;
    return oThisTable.rows[0].cells.length;
}

/* Create a jQuery options object, with one property for each table column
 *   PARAMETERS:
 *     numCols  = number of columns in table
 *     optArray = array with even number of elements; the array elements are paired beginning with elements 0 and 1.  The first element of the pair is
   the column index.  The 2nd element in the pair is an object of properties(options) corresponding with the previous column index.
 */
function setColumnOptions ( numCols, optArray )
{
    var array = new Array(numCols);
    for (i=0; i<array.length; i++) { array[i] = null; }
    if ( numCols <= 0  || optArray.length % 2 != 0 ) { return array; }
    
    for ( var i = 0; i < numCols; i++ ) { // array index
        for ( var j = 0; j < optArray.length; j = j + 2 ) { 
            // j is an optArray col index
            var colIndex = optArray[j];
            if ( colIndex == i ) {
                // store optArray options for this column
                array[i] = optArray[j+1]
            }
        }
    }
    return array;
}

/* Return the number of prepended columns on the table.  
 *
 * nInTable = a node inside the .tableSet of interest
 */
function fnGetNumPrependCols ( nInTableSet )
{
    var iPrependColNum = $(nInTableSet).closest('.tableSet').find('.responseTable col.prependCol').length;
    return iPrependColNum;
}

/* Function to check the time elapsed since some earlier point in the program.  Returns the current time in a Date object.  The initial call (with no input parameter) will only return the current time.  Subsequent calls (using the previous return value as the input parameter) will output the elapsed time in an alert box.
 */
function fnCheckTime( startDate ) 
{
    if ( ! startDate ) 
    { return ( new Date() ); }
    else 
    {
        var time1 = startDate.getTime();
        var time2 = ( new Date() ).getTime();
        alert('Elapsed time = ' + (time2 - time1) + ' ms');
        return time2;
    }
}