// Hide the sesame response table until it is used.
$(document).ready( function () { 
    $('div#sesameResponse').hide();
});

function sesame(identifier)
{
    if ( typeof(identifier) == "undefined" || identifier == null ) 
    { 
        identifier = $('input#lookup').val(); 
        $('span#queryIdentifier')[0].name = identifier;
    }
    var xmlHttp; // The AJAX request object
    var url = "crossmatch?SESAME="; 
    
    setSesameStatusMessage("querying for '" + identifier + "' ...");
    initializeAjax();
    xmlHttp.onreadystatechange = function () { requestVerify(xmlHttp); }
    xmlHttp.open('GET', url + identifier , true);
    xmlHttp.send('');
    return xmlHttp;
    
    /* Set the sesame query status message. */
    function setSesameStatusMessage(message)
    {
        $('span#lookUpStatusMessage').text(message);
    }
    
    /* Instantiate the AJAX request object. */
    function initializeAjax() 
    {
        // Firefox, Opera 8.0+, Safari
        try 
        { 
            xmlHttp=new XMLHttpRequest(); 
        }
        catch (e) 
        {
            // Internet Explorer
            try { xmlHttp=new ActiveXObject("Msxml2.XMLHTTP"); }
            catch (e) 
            {
                try { xmlHttp=new ActiveXObject("Microsoft.XMLHTTP"); }
                catch (e) 
                {
                    throw "Your browser does not support AJAX!";
                }
            }
        }
    }
    
    /* Verify that the request finished properly. */
    function requestVerify(req) 
    {
        // only if req shows "complete"
        if (req.readyState == 4) 
        {
            // only if "OK"
            if (req.status == 200) 
            {
                // ...processing statements go here...
                handleResponse();
            } 
            else 
            {
                throw("There was a problem updating the test results:\n" + req.statusText);
            }
        }
    }

    /* Handle the server's AJAX response. 
     * This is where I will process the returned XML.
     */
    function handleResponse() 
    {
        var xmlDoc = xmlHttp.responseXML;
        
        setSesameStatusMessage("");
        
        // Get first set of coords.
        var jpos = $(xmlDoc).find('jpos:eq(0)').text();
        var asRaDec = parseCoords(jpos);
        writeOutputTable(xmlDoc);
        setCoords(asRaDec.ra, asRaDec.dec, "0.1");
    }
}

/* Fill in the sesame response output table.  Proceed left-to-right, 
 * top-to-bottom.
 */
function writeOutputTable(xmlDoc)
{
    var nTable = $('table#sesameResponseTable');
    // Simbad data
    var nSimbad;
    // Ned data
    var nNed;
    // Get XML DOM nodes for simbad and ned resolvers
    $(xmlDoc).find('Resolver[name]').each( function () {
        var sResolverName = $(this).attr('name'); // name value
        var rSimbadPattern = /Simbad/;
        var rNedPattern = /NED/;
        // If name value matches pattern, set node
        if ( rSimbadPattern.test(sResolverName) ) { nSimbad = $(this); }
        else if ( rNedPattern.test(sResolverName) ) { nNed = $(this); }
    });
    // Fill each row of table
    nTable.find('tbody tr').each( function () {
        sSpanClass = $(this).find('span')[0].className;
        setSimbadNedNodes( sSpanClass );
    });
    
    $('div#sesameResponse').show("normal");
    
    /* Fill in HTML table cells for a given XML tag name; do this for
     * Simbad and NED data. */
    function setSimbadNedNodes(sTagName)
    {
        var bSimbadHasData = true;
        var bNedHasData = true;
        // Show row by default
        nSimbad.find(sTagName).closest('tr').show();

        // Find Span elements with class sTagName
        anSpan = nTable.find( '.' + sTagName );
        // First Span is Simbad
        anSpan[0].innerHTML = nSimbad.find(sTagName).text();
        if ( anSpan[0].innerHTML.length == 0 )
        {
            bSimbadHasData == false;
        }
        // Second Span is Ned
        anSpan[1].innerHTML = nNed.find(sTagName).text();
        if ( anSpan[1].innerHTML.length == 0 )
        {
            if (! bSimbadHasData ) 
            {
                // If neither Simbad nor Ned have data, hide row
                nNed.find(sTagName).closest('tr').hide()
            }
        }
    }
    
    /*  */
    function getXmlData(nResolver, sSelectorText)
    {
        return nResolver.find(sSelectorText).text()
    }
}

// Set coords from table
function setCoordsFromTable(nClickedElement)
{
    // Get the coord string
    var sCoords = $(nClickedElement).find('span.jpos').text();
    // Parse and write in form
    var oCoords = parseCoords(sCoords);
    setCoords(oCoords.ra, oCoords.dec, "");
}

/* Parse a string of RA and Dec coordinates.
 * Return RA and Dec as array elements.
 */
function parseCoords(coordString)
{
    // one or more non-space characters
    var pattern = /\S+/g; 
    pattern.lastIndex = 0; // start from beginning of string
    var ra = pattern.exec(coordString);
    var dec = pattern.exec(coordString);
    return { "ra":ra , "dec":dec};
}
        
